unit DuplicateDirectoryModule;

interface

uses
	Windows, SysUtils, ShellApi, ActiveX,
	LInterfaces, EOFileIntf;

type
	TDuplicateDirectoryAction = class(TInterfacedObject, IAction, IObserver)
	private
		FEnabled: Boolean;
		{IActionItem}
		function GetText: WideString;
		function GetChecked: Boolean;
		function GetItems: IActionItems;
		function Execute: Boolean;
		{IAction}
		function GetName: string;
		function GetHint: WideString;
		function GetEnabled: Boolean;
		{IObserver}
		procedure Observe;
	end;

	TGlobalExtension = class(TInterfacedObject, IExtension, IExtensionInfo, IThreadHandler)
	private
		{IExtension}
		procedure Initialize;
		procedure Finalize;
		{IExtensionInfo}
		function GetName: WideString;
		procedure Dialog;
		{IThreadHandler}
		procedure ThreadBegun(const AThread: IInterface);
		procedure ThreadEnded(const AThread: IInterface);
	end;

implementation

function ShellCopy(const FileName, ToFileName: WideString): Boolean;
var
	Op: SHFILEOPSTRUCTW;
	Fr2, To2: WideString;
begin
	Fr2 := FileName + #0;
	To2 := ToFileName + #0;

	FillChar(Op, SizeOf(Op), 0);
	Op.Wnd    := GetActiveWindow;
	Op.wFunc  := FO_COPY;
	Op.pFrom  := PWideChar(Fr2);
	Op.pTo    := PWideChar(To2);
	Op.fFlags := FOF_RENAMEONCOLLISION;

	Result := SHFileOperationW(Op) = 0;
end;

{ TDuplicateDirectoryAction }

function TDuplicateDirectoryAction.Execute: Boolean;
var
	View: IView;
	SourceDir: WideString;
begin
	View := (GetCurrentThread as IEOFileThread).ActiveView;

	SourceDir := View.FocusedFile^.Name;
	if Pos('\', SourceDir) = 0 then
	begin
		(*
			[g̎ ex. View.Model.Path.Directory = 'D:\'
			D:\\DirName݂ɂȂ邯SHFileOperation͎󗝂Ă(蔲)
		*)
		SourceDir := View.Model.Path.Directory + '\' + SourceDir;
	end;

	ShellCopy(SourceDir, SourceDir + '(2)');

	Result := True;
end;

function TDuplicateDirectoryAction.GetChecked: Boolean;
begin
	Result := False
end;

function TDuplicateDirectoryAction.GetEnabled: Boolean;
begin
	Result := FEnabled
end;

function TDuplicateDirectoryAction.GetHint: WideString;
begin
	Result := 'fBNgd܂'
end;

function TDuplicateDirectoryAction.GetItems: IActionItems;
begin
	Result := nil
end;

function TDuplicateDirectoryAction.GetName: string;
begin
	Result := 'DuplicateDirectory'
end;

function TDuplicateDirectoryAction.GetText: WideString;
begin
	Result := 'fBNgd'
end;

procedure TDuplicateDirectoryAction.Observe;
var
	View: IView;

	function DirFocused: Boolean;
	begin
		if Supports(View.Model, IArchive) then
			Result := False
		else
			Result := (View.Count > 0) and (View.FocusedFile^.Kind = fkFile) and
				(View.FocusedFile^.Attributes and faDirectory <> 0);
	end;

begin
	View := (GetCurrentThread as IEOFileThread).ActiveView;
	FEnabled := DirFocused;
end;

{ TGlobalExtension }

procedure TGlobalExtension.Dialog;
const
	S = 'Tv̂悤Ȃł by YT';
begin
	MessageBoxW(0, S, 'DuplicateDirectory', MB_ICONINFORMATION)
end;

procedure TGlobalExtension.Finalize;
begin
end;

function TGlobalExtension.GetName: WideString;
begin
	Result := 'DuplicateDirectory'
end;

procedure TGlobalExtension.Initialize;
begin
end;

procedure TGlobalExtension.ThreadBegun(const AThread: IInterface);
var
	Action: IAction;
begin
	Action := TDuplicateDirectoryAction.Create;

	//Xbh[JANVo^
	(AThread as IActionManager).Attach(Action);

	//Ďo^
	(AThread as IObservable).Attach(Action as IObserver);
end;

procedure TGlobalExtension.ThreadEnded(const AThread: IInterface);
begin
end;

procedure Register;
begin
	(GetApplication as IExtensionManager).RegisterExtension(TGlobalExtension.Create);
end;

initialization

	if GetInterfaceVersion = EOFileIntf.InterfaceVersion then
		Register;

end.

